@@ -17,7 +17,7 @@ from cosapp.core.numerics.basics import MathematicalProblem, SolverResults
 from cosapp.core.numerics.boundary import Unknown
 from cosapp.drivers.abstractsolver import AbstractSolver
 from cosapp.drivers.optionaldriver import OptionalDriver
-from cosapp.drivers.utils import UnknownAnalyzer, ConstraintParser
+from cosapp.drivers.utils import ConstraintParser, dealias_problem
 from cosapp.recorders.recorder import BaseRecorder
 from cosapp.utils.options_dictionary import OptionsDictionary
 from cosapp.utils.helpers import check_arg
@@ -48,7 +48,7 @@ class Optimizer(AbstractSolver):
     name : str
         Name of the driver
     owner : System, optional
-        :py:class:`~cosapp.systems.system.System` to which this driver belong; default None
+        :py:class:`~cosapp.systems.system.System` to which driver belongs; defaults to `None`
     **kwargs : Any
         Keyword arguments will be used to set driver options
 
@@ -88,19 +88,19 @@ class Optimizer(AbstractSolver):
 
     def __init__(self,
         name: str,
-        owner: "Optional[cosapp.systems.System]" = None,
+        owner: Optional["cosapp.systems.System"] = None,
         **kwargs
     ) -> None:
-        """Initialize a driver
+        """Initialize driver
 
         Parameters
         ----------
         name: str, optional
-            Name of the `Module`
-        owner : System, optional
-            :py:class:`~cosapp.systems.system.System` to which this driver belong; default None
-        **kwargs : Dict[str, Any]
-            Optional keywords arguments
+            Name of the `Driver`.
+        owner: System, optional
+            :py:class:`~cosapp.systems.system.System` to which this driver belong; defaults to `None`.
+        **kwargs:
+            Additional keywords arguments forwarded to base class.
         """
         super().__init__(name, owner, **kwargs)
 
@@ -122,7 +122,7 @@ class Optimizer(AbstractSolver):
             allow_none = True,
         )
         self.options.declare(
-            'eps', 1.5e-08, dtype=float, lower=1.5e-8, upper=1.,
+            'eps', 2**(-26), dtype=float, lower=2**(-26), upper=1.,
             desc="Step size used for numerical approximation of the Jacobian.",
         )
         self.options.declare(
@@ -139,7 +139,7 @@ class Optimizer(AbstractSolver):
         )
         self._filter_options(kwargs, aliases={'tol': 'ftol', 'max_iter': 'maxiter'})
 
-    def set_objective(self, expression: str) -> None:
+    def set_minimum(self, expression: str) -> None:
         """Set the scalar objective function to be minimized.
 
         Parameters
@@ -154,6 +154,35 @@ class Optimizer(AbstractSolver):
         if objective.constant:
             warnings.warn(f"Objective is constant {objective.eval()}")
 
+    def set_maximum(self, expression: str) -> None:
+        """Set the scalar objective function to be maximized.
+
+        Parameters
+        ----------
+        expression : str
+            The objective expression to be maximized.
+        """
+        self.set_minimum(f"-({expression})")
+
+    def set_objective(self, expression: str) -> None:
+        """Set the scalar quantity to be minimized.
+        Same as `set_minimum`.
+
+        Note:
+        -----
+        This method is deprecated, and should be replaced by either
+        `set_minimum` or `set_maximum`, depending on the objective.
+
+        Parameters
+        ----------
+        expression : str
+            The objective expression to be minimized.
+        """
+        warnings.warn(
+            "method `set_objective` is deprecated; use `set_minimum` or `set_maximum` instead."
+        )
+        self.set_minimum(expression)
+
     @staticmethod
     def available_methods() -> List[str]:
         """Returns all possible values of option `method`.
@@ -261,8 +290,7 @@ class Optimizer(AbstractSolver):
         super().setup_run()
         
         # Resolve unknown aliasing and connected unknowns
-        analyzer = UnknownAnalyzer(self.owner)
-        self.problem = analyzer.filter_problem(self._raw_problem)
+        self.problem = dealias_problem(self._raw_problem)
 
     def _fun_wrapper(self, expression: EvalString) -> Callable[[numpy.ndarray], float]:
         """Wrapper around objective and constraint expression to propagate the x values in the
@@ -313,8 +341,8 @@ class Optimizer(AbstractSolver):
                     need_update = True
 
             if need_update:
-                for child in self.exec_order:
-                    self.children[child].run_once()
+                for driver in self.children.values():
+                    driver.run_once()
 
             return expression.eval()
 
@@ -409,12 +437,18 @@ class Optimizer(AbstractSolver):
             output = scipy.optimize.minimize(
                 fresidues, x0,
                 args=args,
+                tol=options['ftol'],
                 method=options['method'],
                 bounds=bounds,
                 constraints=constraints,
                 options=sub_options,
                 callback=callback,
             )
+        if output.fun < self.objective:
+            # Solver solution does not correspond to
+            # last system execution; rerun to synch.
+            self._fresidues(output.x)
+            self._record_data()
         return output
 
     def _precompute(self):
