@@ -18,35 +18,34 @@ class RunOnce(Driver):
     name : str
         Name of the driver
     owner : System, optional
-        :py:class:`~cosapp.systems.system.System` to which this driver belong; default None
+        :py:class:`~cosapp.systems.system.System` to which driver belongs; defaults to `None`
     **kwargs : Any
         Keyword arguments will be used to set driver options
     """
 
     __slots__ = ('initial_values', 'solution')
 
-    def __init__(self,
+    def __init__(
+        self,
         name: str,
-        owner: "Optional[cosapp.systems.System]" = None,
+        owner: Optional["cosapp.systems.System"] = None,
         **kwargs
     ) -> None:
-        """Initialize a driver
+        """Initialize driver
 
         Parameters
         ----------
         name: str, optional
-            Name of the `Module`
-        owner : System, optional
-            :py:class:`~cosapp.systems.system.System` to which this driver belong; default None
-        **kwargs : Dict[str, Any]
-            Optional keywords arguments
+            Name of the `Driver`.
+        owner: System, optional
+            :py:class:`~cosapp.systems.system.System` to which driver belongs; defaults to `None`.
+        **kwargs:
+            Additional keywords arguments forwarded to base class.
         """
         super().__init__(name, owner, **kwargs)
         
-        self.initial_values = dict()  # type: Dict[str, float]
-            # desc="Initial guess for the iteratives."
-        self.solution = dict()  # type: Dict[str, float]
-            # desc="Dictionary (name, value) of the latest solution reached."
+        self.initial_values: Dict[str, Boundary] = dict()  # Initial guess for the iteratives
+        self.solution: Dict[str, float] = dict()  # Dictionary (name, value) of the latest solution reached
 
     def set_init(self, modifications: Dict[str, Any]) -> None:
         """Define initial values for one or more variables.
@@ -76,15 +75,15 @@ class RunOnce(Driver):
         for variable, value in modifications.items():
             boundary = Boundary(self.owner, variable, default=value)
 
-            if boundary.name in self.initial_values:
-                old_boundary = self.initial_values[boundary.name]
-                old_boundary.set_default_value(boundary.default_value, boundary.mask)
-                self.initial_values[boundary.name] = old_boundary
-            else:
-                self.initial_values[boundary.name] = boundary
+            # Check if boundary.name already exists
+            actual = self.initial_values.setdefault(boundary.name, boundary)
 
-            # Set the System with the init value - useful if this driver is not inside a solver
-            self.initial_values[boundary.name].set_to_default()
+            if actual is not boundary:
+                # Update already existing boundary with new default value and mask
+                actual.set_default_value(boundary.default_value, boundary.mask)
+
+            # Set owner system with the init value - useful if this driver is not inside a solver
+            actual.set_to_default()
 
             # Setting a new initial value implies, we will use the init and so the solution is cleared
             self.solution.clear()
@@ -139,23 +138,24 @@ class RunOnce(Driver):
         MathematicalProblem
             The full mathematical problem to solve for the case
         """
-        return self.owner.get_unsolved_problem()
+        return self.owner.assembled_problem()
 
     def setup_run(self):
         """Method called once before starting any simulation."""
         super().setup_run()
-        
-        if not self.owner.is_standalone() and self.owner.parent is None:
-            self.owner.open_loops()  # Force loops opening to test if the owner needs a solver
+        owner = self.owner
 
-            if self.get_problem().shape != (0, 0):
+        if not owner.is_standalone() and owner.parent is None:
+            owner.open_loops()  # Force loops opening to test if the owner needs a solver
+
+            if not self.get_problem().is_empty():
                 logger.warning(
                     "Required iterations detected, not taken into account in {} driver.".format(
                         type(self).__qualname__
                     )
                 )
 
-            self.owner.close_loops()
+            owner.close_loops()
 
     def _precompute(self):
         """Set execution order and start the recorder."""
