@@ -31,28 +31,29 @@ class RunSingleCase(IterativeCase):
     name : str
         Name of the driver
     owner : System, optional
-        :py:class:`~cosapp.systems.system.System` to which this driver belong; default None
+        :py:class:`~cosapp.systems.system.System` to which driver belongs; defaults to `None`
     **kwargs : Any
         Keyword arguments will be used to set driver options
     """
 
     __slots__ = ('__case_values', '__raw_problem', '__processed', 'problem')
 
-    def __init__(self,
+    def __init__(
+        self,
         name: str,
         owner: Optional[System] = None,
         **kwargs
         ) -> None:
-        """Initialize a driver
+        """Initialize driver
 
         Parameters
         ----------
         name: str, optional
             Name of the `Module`
-        owner : System, optional
-            :py:class:`~cosapp.systems.system.System` to which this driver belong; default None
+        owner: System, optional
+            :py:class:`~cosapp.systems.system.System` to which driver belongs; defaults to `None`
         **kwargs : Dict[str, Any]
-            Optional keywords arguments
+            Optional keywords arguments formwarded to base class.
         """
         super().__init__(name, owner, **kwargs)
         self.__case_values = []  # type: List[AssignString]
@@ -85,13 +86,16 @@ class RunSingleCase(IterativeCase):
         self.problem = None
 
     def merge_problems(self) -> None:
-        self.problem = self.merged_problem()
+        # Activate targets in processed problems
+        for problem in self.__processed.problems:
+            problem.activate_targets()
+        self.problem = self.merged_problem(copy=False)
 
-    def merged_problem(self) -> MathematicalProblem:
+    def merged_problem(self, copy=True) -> MathematicalProblem:
         handler = self.__processed
         name = self.name
         try:
-            return handler.merged_problem(name=name, offdesign_prefix=name)
+            return handler.merged_problem(name=name, offdesign_prefix=None, copy=copy)
         except ValueError as error:
             error.args = (f"{error.args[0]} in {name!r}",)
             raise
@@ -101,18 +105,15 @@ class RunSingleCase(IterativeCase):
         super().setup_run()
         
         raw = self.__raw_problem
-        # Force graph analysis by creating new problem handler
-        self.__processed = DesignProblemHandler(self.owner)
-        processed = self.__processed
         # Transfer problem copies from `raw` to `processed`
-        processed.problems = raw.export_problems(prune=False)
+        processed = raw.copy(prune=False)
 
         # Add owner off-design problem to `processed.offdesign`
-        owner_problem = self.owner.get_unsolved_problem()
+        owner_problem = self.owner.assembled_problem()
         processed.offdesign.extend(owner_problem)
 
         # Resolve unknown aliasing in `processed`
-        processed.problems = processed.export_problems()
+        self.__processed = processed.copy(prune=True)
         self.merge_problems()
 
     def add_offdesign_problem(self, offdesign: MathematicalProblem) -> MathematicalProblem:
@@ -125,7 +126,7 @@ class RunSingleCase(IterativeCase):
         """
         # Unknowns & residues are duplicated to avoid side effects between points
         # Existing unknowns and equations are silently overwritten.
-        if offdesign.shape != (0, 0):
+        if not offdesign.is_empty():
             self.__processed.offdesign.extend(offdesign, copy=True, overwrite=True)
             self.merge_problems()
         return self.problem
