@@ -9,7 +9,7 @@ from typing import Tuple, NamedTuple, List, Dict, Union, Optional, Any
 
 from cosapp.core.time import UniversalClock
 from cosapp.ports.enum import PortType
-from cosapp.drivers.driver import Driver
+from cosapp.drivers.driver import Driver, Recorder
 from cosapp.drivers.time.utils import (
     TimeVarManager,
     TimeStepManager,
@@ -18,7 +18,6 @@ from cosapp.drivers.time.utils import (
 from cosapp.drivers.time.scenario import Scenario
 from cosapp.multimode.discreteStepper import DiscreteStepper
 from cosapp.multimode.event import Event
-from cosapp.recorders.recorder import BaseRecorder
 from cosapp.recorders import DataFrameRecorder
 from cosapp.utils.helpers import check_arg
 from cosapp.utils.logging import LogFormat, LogLevel
@@ -49,7 +48,7 @@ class ExplicitTimeDriver(Driver):
 
     def __init__(self,
         name = "Explicit time driver",
-        owner: "Optional[cosapp.systems.System]" = None,
+        owner: Optional["cosapp.systems.System"] = None,
         time_interval: Tuple[float, float] = None,
         dt: float = None,
         record_dt: bool = False,
@@ -169,8 +168,16 @@ class ExplicitTimeDriver(Driver):
         """Scenario: the simulation scenario, defining initial and boundary conditions"""
         return self.__scenario
 
-    def add_recorder(self, recorder: BaseRecorder, period: Number = None) -> BaseRecorder:
-        """Add an internal data recorder storing the time evolution of values of interest"""
+    def add_recorder(self, recorder: Recorder, period: Optional[Number] = None) -> Recorder:
+        """Add an internal recorder storing the time evolution of values of interest.
+
+        Parameters
+        ----------
+        - recorder [BaseRecorder]:
+            The recorder to be added.
+        - period [Number, optional]:
+            Recording period. If `None` (default), data are recorded at all time steps.
+        """
         if 'time' not in recorder:
             cls = type(recorder)
             recorder = cls.extend(recorder, includes='time')
@@ -278,8 +285,12 @@ class ExplicitTimeDriver(Driver):
         manager = self.__var_manager
         owner_transients = manager.problem.transients
         self.__recorded_events = []
+        get_data = lambda var: [
+            copy.copy(var.value),
+            copy.copy(var.d_dt),
+        ]
         tr_data = dict(
-            (key, [var.value, var.d_dt, None, None])
+            (key, [*get_data(var), None, None])
             for key, var in owner_transients.items()
         )
 
@@ -293,7 +304,7 @@ class ExplicitTimeDriver(Driver):
             # Store transient values and derivatives @ t + dt
             for key, var in owner_transients.items():
                 data = tr_data[key]
-                data[2:4] = [var.value, var.d_dt]
+                data[2:4] = get_data(var)
             
             if stepper.event_detected():
                 stepper.set_data(
@@ -348,7 +359,7 @@ class ExplicitTimeDriver(Driver):
 
             record_dt(dt)
             for data in tr_data.values():
-                data[0:2] = copy.copy(data[2:4])
+                data[0:2] = data[2:4]
 
             return next_t, must_stop
 
@@ -390,8 +401,7 @@ class ExplicitTimeDriver(Driver):
 
     def transition(self) -> None:
         """Execute owner system transition and reinitialize sub-drivers"""
-        for system in self.owner.tree():
-            system.transition()
+        self.owner.tree_transition()
         # Reinitialize sub-drivers after system transition
         for driver in self.children.values():
             driver.call_setup_run()
@@ -418,6 +428,7 @@ class ExplicitTimeDriver(Driver):
         logger.debug("Reset rates")
         for rate in self._rates.values():
             rate.reset()
+        self.__stepper.reset()
     
     def __init_modevars(self):
         """Force init value of output mode vars or owner system."""
@@ -438,9 +449,10 @@ class ExplicitTimeDriver(Driver):
             transient.reset()
 
     def _update_children(self) -> None:
-        if len(self.children) > 0:
-            for name in self.exec_order:
-                self.children[name].run_once()
+        """Execute sub-drivers, if any, or owner's subsystem drivers"""
+        if self.children:
+            for driver in self.children.values():
+                driver.run_once()
         else:
             self.owner.run_children_drivers()
 
