@@ -13,7 +13,6 @@ from cosapp.core.numerics.boundary import AbstractTimeUnknown, TimeUnknown
 from cosapp.ports.port import ExtensiblePort
 from cosapp.systems.system import System
 from cosapp.utils.helpers import check_arg
-from cosapp.drivers import Driver
 from cosapp.drivers.time.scenario import TimeAssignString
 
 T = TypeVar('T')
@@ -288,7 +287,7 @@ class TimeVarManager:
         return self.__context
 
     @property
-    def problem(self) -> "MathematicalProblem":
+    def problem(self):
         """Mathematical problem handled by manager"""
         return self.__problem
 
@@ -320,7 +319,7 @@ class TimeVarManager:
     def update_transients(self) -> None:
         """Update the transient variable dictionary (see property `transients` for details)"""
         context = self.__context
-        problem = context.get_unsolved_problem()
+        problem = context.assembled_problem()
         context_transients = problem.transients
         ders = dict()
         reference2name = dict()
@@ -564,7 +563,7 @@ class SystemInterpolator:
         check_arg(driver, 'driver', ExplicitTimeDriver)
         self.__owner = driver
         self.__system = system = driver.owner
-        problem = system.get_unsolved_problem()
+        problem = system.assembled_problem()
         self.__transients = transients = problem.transients
         self.__interp = dict.fromkeys(transients, None)
 
@@ -584,10 +583,9 @@ class SystemInterpolator:
 
     @interp.setter
     def interp(self, interp: Dict[str, Callable]):
-        transients = self.__transients
         check_arg(
             interp, "interp", dict,
-            lambda d: set(d) == set(transients)
+            lambda d: set(d) == set(self.__transients)
         )
         context = self.system
         for key, func in interp.items():
