@@ -1,6 +1,3 @@
-"""
-Classes containing all `System` variables.
-"""
 import logging
 import copy
 from collections import OrderedDict
@@ -48,12 +45,13 @@ class BasePort(visitor.Component):
         """
         if not isinstance(direction, PortType):
             raise TypeError(f"Direction must be PortType; got {direction}.")
+        from cosapp.systems import System
 
-        self._variables = OrderedDict()  # type: Dict[str, BaseVariable]
-        self._name = self._name_check(name)  # type: str
-        self._direction = direction  # type: PortType
-        self._owner = None  # type: Optional[cosapp.systems.System]
-        self.__clearance = None
+        self._variables: Dict[str, BaseVariable] = OrderedDict()
+        self._name: str = self._name_check(name)
+        self._direction: PortType = direction
+        self._owner: Optional[System] = None
+        self.__clearance: Scope = None
         self.scope_clearance = Scope.PRIVATE
 
     def accept(self, visitor: visitor.Visitor) -> None:
@@ -61,7 +59,7 @@ class BasePort(visitor.Component):
         visitor.visit_port(self)
 
     @property
-    def owner(self) -> "Optional[cosapp.systems.System]":
+    def owner(self) -> Optional["cosapp.systems.System"]:
         """System : `System` owning the port."""
         return self._owner
 
@@ -391,7 +389,8 @@ class BasePort(visitor.Component):
             criterion = lambda name: False
         else:
             read_only = [
-                n for n, details in self._variables.items() if details.scope > self.__clearance
+                name for (name, variable) in self._variables.items()
+                if variable.scope > self.__clearance
             ]
             criterion = lambda name: not self._owner.is_running() and name in read_only
         self.__out_of_scope = criterion
@@ -509,10 +508,11 @@ class BasePort(visitor.Component):
         else:
             return self._variables[name].get_validity_comment(status)
 
-    def copy(self,
+    def copy(
+        self,
         name: Optional[str] = None,
         direction: Optional[PortType] = None,
-    ) -> "BasePort":
+    ):
         """Duplicates the port.
 
         Parameters
@@ -793,14 +793,13 @@ class Port(BasePort):
         variables : Dict[str, Any], optional
             Dictionary of variables with their value and details; default: None = default value
         """
-        check_arg(variables, "variables", (type(None), dict))
-
         super().__init__(name, direction)
         self._locked = False  # type: bool
 
         self.setup()
 
         if variables is not None:
+            check_arg(variables, "variables", dict)
             for name, value in variables.items():
                 variable_value = value
                 if isinstance(value, dict):
@@ -909,10 +908,11 @@ class Port(BasePort):
             scope=scope,
         )
 
-    def copy(self, 
+    def copy(
+        self, 
         name: Optional[str] = None,
         direction: Optional[PortType] = None,
-    ) -> "BasePort":
+    ):
         """Duplicates the port.
 
         Parameters
@@ -927,12 +927,12 @@ class Port(BasePort):
         Port
             The copy of the current port
         """
-        new_name = self.name if name is None else name
-        new_direction = self.direction if direction is None else direction
-
-        new_port = type(self)(new_name, new_direction)
+        cls = type(self)
+        new_port = cls(
+            name or self.name,
+            direction or self.direction,
+        )
         new_port.set_from(self, copy.copy)
-
         return new_port
 
     def set_from(self,
