@@ -24,7 +24,7 @@ class NestedQuad(System):
         foo = self.add_child(Quadratic('foo'), pulling={'k': 'c'})
         bar = self.add_child(Quadratic('bar'))
 
-        self.connect(foo.outwards, bar.inwards, {'y': 'k'})
+        self.connect(foo, bar, {'y': 'k'})
 
 
 def test_design_unknown_aliasing_1(caplog):
@@ -48,7 +48,7 @@ def test_design_unknown_aliasing_1(caplog):
     
     assert len(caplog.records) > 0
     assert re.match(
-        "Replace unknown 'foo.inwards.k' by 'inwards.c'",
+        "Replace unknown 'foo.k' by 'c'",
         caplog.records[0].message
     )
     assert top.foo.k == pytest.approx(1.5)
@@ -85,7 +85,7 @@ def test_design_unknown_aliasing_2(caplog):
     assert len(caplog.records) > 0
     assert any(
         re.match(
-            "Unknown 'foo.inwards.k' is aliased by 'top.inwards.c', defined outside the context of 'foo'",
+            "Unknown 'foo.k' is aliased by 'top.c', defined outside the context of 'foo'",
             record.message
         )
         for record in caplog.records
@@ -115,7 +115,33 @@ def test_design_unknown_aliasing_3(caplog):
     
     assert len(caplog.records) > 0
     assert re.match(
-        "Skip connected unknown 'sub2.bar.inwards.k'",
+        "Skip connected unknown 'sub2.bar.k'",
+        caplog.records[0].message
+    )
+
+
+def test_design_unknown_aliasing_4(caplog):
+    """Same as `test_design_unknown_aliasing_1`, with
+    one additional hierarchical level.
+    """
+    top = System('top')
+    sub = top.add_child(NestedQuad('sub'))
+    solver = top.add_driver(NonLinearSolver('solver', tol=1e-9))
+    solver.add_equation('sub.foo.y == 7').add_unknown('sub.foo.k')
+
+    sub.foo.x = 2.0
+    sub.bar.x = 0.1
+    caplog.clear()
+    with caplog.at_level(logging.INFO):
+        top.run_drivers()
+    
+    assert sub.foo.k == pytest.approx(1.5)
+    assert sub.foo.y == pytest.approx(7)
+    assert sub.bar.k == sub.foo.y
+    assert sub.foo.k == sub.c
+    assert len(caplog.records) > 0
+    assert re.match(
+        "Replace unknown 'sub.foo.k' by 'sub.c'",
         caplog.records[0].message
     )
 
@@ -135,7 +161,7 @@ def test_design_connected_unknown_1(caplog):
     
     assert len(caplog.records) > 0
     assert re.match(
-        "Skip connected unknown 'bar.inwards.k'",
+        "Skip connected unknown 'bar.k'",
         caplog.records[0].message
     )
 
@@ -158,7 +184,7 @@ def test_design_connected_unknown_2(caplog):
     
     assert len(caplog.records) > 0
     assert re.match(
-        "Skip connected unknown 'inwards.k'",
+        "Skip connected unknown 'k'",
         caplog.records[0].message
     )
 
@@ -182,14 +208,14 @@ def test_design_offdesign_aliasing_1(caplog):
     solver.runner.design.add_equation('foo.y == 7').add_unknown('foo.k')
 
     caplog.clear()
-    with pytest.raises(ValueError, match="'inwards.c' is defined as design and off-design unknown"):
+    with pytest.raises(ValueError, match="'c' is defined as design and off-design unknown"):
         with caplog.at_level(logging.INFO):
             top.run_drivers()
     
     assert len(caplog.records) > 0
     assert any(
         re.match(
-            "Replace unknown 'foo.inwards.k' by 'inwards.c'",
+            "Replace unknown 'foo.k' by 'c'",
             record.message
         )
         for record in caplog.records
@@ -215,14 +241,14 @@ def test_design_offdesign_aliasing_2(caplog):
     solver.runner.design.add_equation('foo.y == 7').add_unknown('c')
 
     caplog.clear()
-    with pytest.raises(ValueError, match="'inwards.c' is defined as design and off-design unknown"):
+    with pytest.raises(ValueError, match="'c' is defined as design and off-design unknown"):
         with caplog.at_level(logging.INFO):
             top.run_drivers()
     
     assert len(caplog.records) > 0
     assert any(
         re.match(
-            "Replace unknown 'foo.inwards.k' by 'inwards.c'",
+            "Replace unknown 'foo.k' by 'c'",
             record.message
         )
         for record in caplog.records
@@ -248,7 +274,7 @@ def test_design_offdesign_aliasing_3(caplog):
     solver.add_equation('foo.y == 7').add_unknown('foo.k')
 
     caplog.clear()
-    with pytest.raises(ValueError, match="'inwards.c' is defined as design and off-design unknown"):
+    with pytest.raises(ValueError, match="'c' is defined as design and off-design unknown"):
         with caplog.at_level(logging.INFO):
             top.run_drivers()
             # print(solver.problem)
@@ -256,7 +282,7 @@ def test_design_offdesign_aliasing_3(caplog):
     assert len(caplog.records) > 0
     assert any(
         re.match(
-            "Replace unknown 'foo.inwards.k' by 'inwards.c'",
+            "Replace unknown 'foo.k' by 'c'",
             record.message
         )
         for record in caplog.records
@@ -295,7 +321,7 @@ def test_connected_unknown_changing_conf(caplog):
     
     assert len(caplog.records) > 0
     assert re.match(
-        "Skip connected unknown 'bar.inwards.x'",
+        "Skip connected unknown 'bar.x'",
         caplog.records[0].message
     )
 
